@@ -17,11 +17,6 @@
 #ifndef DEFS_H
 #define DEFS_H
 
-/*
- * units used for MTC clock.
- */
-#define MTC_SEC			2400	/* 1 second is 2400 ticks */
-
 /*
  * limits
  */
