@@ -54,3 +54,27 @@ _sndio_parsetype(const char *str, char *type)
 		return NULL;
 	return str;
 }
+
+const char *
+_sndio_parsenum(const char *str, unsigned int *num, unsigned int max)
+{
+	const char *p = str;
+	unsigned int dig, maxq, maxr, val;
+
+	val = 0;
+	maxq = max / 10;
+	maxr = max % 10;
+	for (;;) {
+		dig = *p - '0';
+		if (dig >= 10)
+			break;
+		if (val > maxq || (val == maxq && dig > maxr))
+			return NULL;
+		val = val * 10 + dig;
+		p++;
+	}
+	if (p == str)
+		return NULL;
+	*num = val;
+	return p;
+}
