@@ -135,7 +135,7 @@ dev_sio_open(struct dev *d)
 
 #ifdef DEBUG
 	/*
-	 * We support any parameters combination exposed by the kernel,
+	 * We support any parameter combination exposed by the kernel,
 	 * and we have no other choice than trusting the kernel for
 	 * returning correct parameters. But let's check parameters
 	 * early and nicely report kernel bugs rather than crashing
@@ -152,7 +152,7 @@ dev_sio_open(struct dev *d)
 	if (par.bps > SIO_BPS(BITS_MAX)) {
 		log_puts(d->path);
 		log_puts(": ");
-		log_putu(par.bits);
+		log_putu(par.bps);
 		log_puts(": unsupported sample size\n");
 		goto bad_close;
 	}
@@ -353,11 +353,6 @@ dev_sio_run(void *arg)
 			n = sio_read(d->sio.hdl, data, d->sio.todo);
 			d->sio.todo -= n;
 #ifdef DEBUG
-			if (log_level >= 3 &&
-			    n == 0 && data == base && !sio_eof(d->sio.hdl)) {
-				dev_log(d);
-				log_puts(": read blocked at cycle start\n");
-			}
 			if (log_level >= 4) {
 				dev_log(d);
 				log_puts(": read ");
@@ -437,11 +432,6 @@ dev_sio_run(void *arg)
 			n = sio_write(d->sio.hdl, data, d->sio.todo);
 			d->sio.todo -= n;
 #ifdef DEBUG
-			if (log_level >= 3 &&
-			    n == 0 && data == base && !sio_eof(d->sio.hdl)) {
-				dev_log(d);
-				log_puts(": write blocked at cycle start\n");
-			}
 			if (log_level >= 4) {
 				dev_log(d);
 				log_puts(": wrote ");
